#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <string.h>
#include <linux/cdrom.h>

#include "cdindex.h"
#include "sha.h"
#include "base64.h"

void hash_test(void)
{
   SHA1_CTX       sha;
   unsigned char  digest[20], *base64;
   unsigned long  size;

   SHA1Init(&sha);
   SHA1Update(&sha, (unsigned char *)"0123456789", 10);
   SHA1Final(digest, &sha);

   base64 = rfc822_binary(digest, 20, &size);
   if (strncmp((char *)base64, "h6zsF82dzSCnFsws9nQXtxyKcBY-", size))
   {
       free(base64);
       fprintf(stderr, "Houston. We have a problem.\n");
       fprintf(stderr, "This program doesn't work on your architecture :(\n");
       exit(0);
   }
   free(base64);
}

void calculate(int first, int last, int offsets[100], char id[33])
{
   SHA1_CTX sha;
   unsigned char digest[20], *base64;
   long int size;
   char temp[9];
   int i;

   SHA1Init(&sha);

   /* Before we ran the hash on the binary data, but now to make
    * sure people on both types of endian systems create the same
    * keys, we convert the data to hex-ASCII first. :-)
    */
   sprintf(temp, "%02X", first);
   SHA1Update(&sha, (unsigned char *)temp, strlen(temp));

   sprintf(temp, "%02X", last);
   SHA1Update(&sha, (unsigned char *)temp, strlen(temp));

   for(i = 0; i < 100; i++)
   {
       sprintf(temp, "%08X", offsets[i]);
       SHA1Update(&sha, (unsigned char *)temp, strlen(temp));
   }
   SHA1Final(digest, &sha);

   base64 = rfc822_binary(digest, 20, &size);
   memcpy(id, base64, size);
   id[size] = 0;
   free(base64);
}

static int ReadTOCHeader(int fd, int *first, int *last)
{
   struct cdrom_tochdr tochdr;

   int ret = ioctl(fd, CDROMREADTOCHDR, &tochdr);

   if (!ret)
   {
      *first = tochdr.cdth_trk0;
      *last  = tochdr.cdth_trk1;
   }

   return ret;
}

static int ReadTOCEntry(int fd, int track, int *lba, int *control, int *adr)
{
   int ret;
   struct cdrom_tocentry tocentry;
   tocentry.cdte_track = track;
   tocentry.cdte_format = CDROM_LBA;

   ret = ioctl(fd, CDROMREADTOCENTRY, &tocentry);
   if (!ret)
   {
      assert (tocentry.cdte_format == CDROM_LBA);
      *lba = tocentry.cdte_addr.lba;
      *control = tocentry.cdte_ctrl;
      *adr = tocentry.cdte_adr;
   }

   return ret;
}

int ponder(int *first, int *last, int offsets[100])
{
   int fd; /* CD file descriptor */
   int i, lba, control, adr;
   char *device="/dev/cdrom";

   fd = open(device, O_RDONLY);
   if (fd < 0)
   {
       fprintf(stderr, "Cannot open '%s'\n", device);
       return -1;
   }

   for (i = 0; i < 100; ++i) {
      offsets[i] = 0;
   }

   if (ReadTOCHeader(fd, first, last))
   {
      /* fprintf(stderr, "Cannot read table of contents.\n"); */
      close(fd);
      return -1;
   }

   ReadTOCEntry(fd, CDROM_LEADOUT, &lba, &control, &adr);
   offsets[0] = lba + 150;
   for (i = *first; i <= *last; i++)
   {
      ReadTOCEntry(fd, i, &lba, &control, &adr);
      offsets[i] = lba + 150;
   }

   close(fd);
   return 0;
}

void submit(int first, int last, int offsets[100])
{
   char *home;
   char id[33], url[URL_SIZE], url2[URL_SIZE];
   char toc_string[URL_SIZE], lockfile[255];
   int i;
   struct stat sb;

   calculate(first, last, offsets, id);
   sprintf(toc_string, "%d+%d+%d", first, last, offsets[0]);
   for (i = first; i <= last; i++)
      sprintf(toc_string + strlen(toc_string), "+%d", offsets[i]);

   sprintf(url, "http://www.cdindex.org/cgi-bin/cdi/submit.pl?"
               "id=%s&tracks=%d&toc=%s", id, (last - first) + 1, toc_string);

   sprintf(url2, "openURL(%s)", url);

   printf("Opening netscape to: '%s'\n(Please open it by hand if the "
         "browser doesn't start automatically)\n", url);

   home=getenv("HOME");
   if(!home) home="/";
   sprintf(lockfile,"%.200s/.netscape/lock",home);
   if(fork() == 0) {
     if(lstat(lockfile, &sb)!=-1) {
        execlp("netscape", "netscape", "-remote", url2, NULL);
     } else {
        execlp("netscape", "netscape", url, NULL);
     }
   }
}

void request(int first, int last, int offsets[100], char *path)
{
   char id[33];

   calculate(first, last, offsets, id);
   sprintf(path,
     "GET http://www.cdindex.org/cgi-bin/cdi/get.pl?id=%s HTTP/1.0\n\n", id);
}
