/*  gcdindex CDindex client
 *  Copyright (C) 1999 Nick Lamb <njl195@zepler.org.uk>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdlib.h>
#include <sys/socket.h>
#include <string.h>
#include "gladesrc.h"
#include "gladesig.h"
#include "cdindex.h"
#include "http.h"

GtkWidget *main_window, *artist_entry, *title_entry;
GtkWidget *track_list, *submit_button, *statusbar;
GList *tracks= NULL; /* List of tracks :) */
int first_track, last_track, track_offsets[100];
gint status = 0xface;
char proxyserver[128] = "";
int proxyport;

int
main (int argc, char *argv[])
{
     char *portsep;

  hash_test();

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  main_window = create_main_window ();
  gtk_widget_show (main_window);

  artist_entry= get_widget(main_window, "artist_entry");
  title_entry= get_widget(main_window, "title_entry");
  track_list= get_widget(main_window, "track_list");
  submit_button = get_widget(main_window, "submit_button");
  statusbar = get_widget(main_window, "statusbar");

  /* quickie hack for using a proxy server */
  if(argc == 3 && !strcmp(argv[1], "-p") && strlen(argv[2]) < 128) {
       strcpy(proxyserver, argv[2]);
       portsep = strchr(proxyserver, ':');
       if(portsep) {
	    *portsep = 0;
	    proxyport = atoi(portsep + 1);
       }
  }

  status= gtk_statusbar_push(GTK_STATUSBAR(statusbar), status, "CDindex");
  update_info();
  gtk_main ();
  return 0;
}


void
finish                                 (GtkObject       *object,
                                        gpointer         user_data)
{
gtk_main_quit();
}


void
refresh_clicked                        (GtkButton       *button,
                                        gpointer         user_data)
{
  update_info();
}

void update_info(void) {
  GtkWidget* ptr;
  int i, sd;
  char buffer[1024];

  gtk_list_remove_items(GTK_LIST(track_list), tracks);

  gtk_statusbar_pop(GTK_STATUSBAR(statusbar), status);
  gtk_statusbar_push(GTK_STATUSBAR(statusbar), status,
             "Reading the table of contents from the CD...");
  while (gtk_events_pending()) gtk_main_iteration();

  tracks= NULL;
  if (!ponder(&first_track, &last_track, track_offsets)) {
    gtk_statusbar_pop(GTK_STATUSBAR(statusbar), status);
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), status,
               "Looking up this CD in CDindex...");
    while (gtk_events_pending()) gtk_main_iteration();

    request(first_track, last_track, track_offsets, buffer);

    if(strlen(proxyserver))
	 sd = opensocket(proxyserver, proxyport);
    else
	 sd = opensocket("www.cdindex.org", 80);

    send(sd, buffer, strlen(buffer), 0);

    while ((i= fgetsock(buffer, 1023, sd)) > 0) {
      strtok(buffer, " ");

      if (!strcmp("Artist:", buffer)) {
        gtk_entry_set_text(GTK_ENTRY(artist_entry), strtok(NULL, ""));
      } else if (!strcmp("Album:", buffer)) {
        gtk_entry_set_text(GTK_ENTRY(title_entry), strtok(NULL, ""));
      } else if (!strcmp("Tracks:", buffer)) {
        /* Just throw this away */
      } else if (!strncmp("Track", buffer, 5)) {
        ptr= gtk_list_item_new_with_label(strtok(NULL, ""));
        tracks= g_list_append(tracks, ptr);
      } 
    }

    if (g_list_length(tracks) == 0) {
      gtk_widget_set_sensitive(submit_button, TRUE);
      gtk_entry_set_text(GTK_ENTRY(artist_entry), "Unknown");
      gtk_entry_set_text(GTK_ENTRY(title_entry), "Unknown");
      gtk_statusbar_pop(GTK_STATUSBAR(statusbar), status);
      gtk_statusbar_push(GTK_STATUSBAR(statusbar), status,
                 "Use the submit button to tell CDindex about this CD");
    } else {
      gtk_widget_set_sensitive(submit_button, FALSE);
      gtk_list_append_items(GTK_LIST(track_list), tracks);
      gtk_widget_show_all(track_list);
      gtk_statusbar_pop(GTK_STATUSBAR(statusbar), status);
      gtk_statusbar_push(GTK_STATUSBAR(statusbar), status,
                 "Thanks for using CDindex");
    }
  } else {
    gtk_widget_set_sensitive(submit_button, FALSE);
    gtk_entry_set_text(GTK_ENTRY(artist_entry), "None");
    gtk_entry_set_text(GTK_ENTRY(title_entry), "None");
    gtk_statusbar_pop(GTK_STATUSBAR(statusbar), status);
    gtk_statusbar_push(GTK_STATUSBAR(statusbar), status,
               "Please insert an audio CD to use CDindex");
  }

}

void
submit_clicked                         (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_set_sensitive(submit_button, FALSE);

  if (!ponder(&first_track, &last_track, track_offsets)) {
    submit(first_track, last_track, track_offsets);
  }
}
